'use strict';

var _get = require('babel-runtime/helpers/get')['default'];

var _inherits = require('babel-runtime/helpers/inherits')['default'];

var _createClass = require('babel-runtime/helpers/create-class')['default'];

var _classCallCheck = require('babel-runtime/helpers/class-call-check')['default'];

var _Symbol = require('babel-runtime/core-js/symbol')['default'];

var _Map = require('babel-runtime/core-js/map')['default'];

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default')['default'];

Object.defineProperty(exports, '__esModule', {
  value: true
});

var _instance = require('./instance');

var _instance2 = _interopRequireDefault(_instance);

var _componentEmitter = require('component-emitter');

var _componentEmitter2 = _interopRequireDefault(_componentEmitter);

var _dispatcher = require('./dispatcher');

var _dispatcher2 = _interopRequireDefault(_dispatcher);

var _ActionHandler = _Symbol();
var _Emitter = _Symbol();
var _Registered = _Symbol();
var _Symbols = _Symbol();
var _Ctx = _Symbol();

/**
 * @exports default
 * @class StoreBase
 * @extends Instance
 * @extends Emitter
 *    Base store class for implementing data stores.
 */

var Store = (function (_Instance) {
  _inherits(Store, _Instance);

  function Store() {
    var _this = this;

    _classCallCheck(this, Store);

    _get(Object.getPrototypeOf(Store.prototype), 'constructor', this).call(this);
    this[_Emitter] = new _componentEmitter2['default']();
    this[_Symbols] = new _Map();

    //bind main action handler with bounded this in the scope
    this[_ActionHandler] = function (payload) {
      //run the actual action handlers using the action as the accessor
      var s = _this[_Symbols].get(payload.action);
      if (typeof _this[s] === 'function') {
        _this[s](payload.payload);
      }
    };
  }

  _createClass(Store, [{
    key: 'bindHandler',
    value: function bindHandler(action, handler) {
      if (!this[_Symbols].has(action)) {
        this[_Symbols].set(action, _Symbol());
      }
      this[this[_Symbols].get(action)] = handler;
    }
  }, {
    key: 'dehydrate',

    /**
     * @function
     *  Returns the current state in JS object form.
     */
    value: function dehydrate() {}

    /**
     * @function
     *  @param {Object} state - The state object to fill into the data store.
     *  Fill the data store with the state object, the underlying data is immutable.
     */
  }, {
    key: 'rehydrate',
    value: function rehydrate(state) {}
  }, {
    key: 'changed',
    value: function changed() {
      this[_Emitter].emit('change');
    }
  }, {
    key: 'on',
    value: function on(fn) {
      this[_Emitter].on('change', fn);
    }
  }, {
    key: 'off',
    value: function off(fn) {
      this[_Emitter].off('change', fn);
    }
  }, {
    key: 'ctx',
    get: function get() {
      return this[_Ctx];
    }
  }], [{
    key: 'getInstance',
    value: function getInstance(ctx) {
      var self = _get(Object.getPrototypeOf(Store), 'getInstance', this).call(this, ctx);

      if (!self[_Registered]) {
        if (!ctx.init) {
          //rehydrate data if ctx has not been initialized
          self.rehydrate(ctx.storeData);
        }
        ctx.stores.add(self);
        _dispatcher2['default'].getInstance(ctx).register(self, self[_ActionHandler]);
        self[_Ctx] = ctx;
        self[_Registered] = true;
      }
      return self;
    }
  }]);

  return Store;
})(_instance2['default']);

exports['default'] = Store;
module.exports = exports['default'];
//# sourceMappingURL=store.js.map
